/******************************************************************************/
/*          ______  ___   ______  ___   __________   ______                   */
/*          ___   |/  /   ___   |/  /   ___  ____/   __<  /                   */
/*          __  /|_/ /    __  /|_/ /    __  /_       __  /                    */
/*          _  /  / /     _  /  / /     _  __/       _  /                     */
/*          /_/  /_/      /_/  /_/      /_/          /_/                      */
/*                                                                (2009/2010) */
/******************************************************************************/
/*  Program ucitava tocke (x,f(x)) te radi polinomnu, racionalnu i spline     */
/*  interpolaciju te sprema rezultate u datotekama                            */
/*  Potrebno je mijenjati:                                                    */
/*    - izbrisati pomocnu funkciju f(x), njenu derivaciju fd(x) te njihove    */
/*      pozive ako se koriste vrijednosti nepoznate funkcije jer dio programa */
/*      oznacen kao POMOCNI DIO PROGRAMA mora biti izbrisan u tom slucaju     */
/*    - vrijednosti nepoznate funkcije potrebno je pohraniti u ulaznu datoteku*/
/*      pocevsi ih upisivati od treceg retka                                  */
/*    - broj poznatih vrijednosti (NP)                                        */
/*    - ukupni broj vrijednosti koje racunamo, odnosno interpoliramo (IP)     */
/*                                                                            */
/*  UPOZORENJJE: Ako dolazi do pucanjs tijekom interpolacije, najcesci je     */
/*               uzrok racionalna interpolacija pa je treba ukloniti ili      */
/*               promijeniti granice podrucja                                 */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>   // Definirane standardne ulazno, izlazne funkcije
#include <math.h>    // Definirane potrebne matematicke funkcije
#include "nrutil.c"  // Definirana klasa potrebnih polja i funkcija za njih
#include "polint.c"  // Definirana polinomna interpolacija
#include "ratint.c"  // Definirana interpolacija racionalnim funkcijama
#include "splint.c"  // Definirana spline interpolacija
#include "spline.c"  // Definirana spline interpolacija

#define MAXSTR 80    // Maksimalna duljina stringa
#define NP      6    // Broj tocaka u kojima poznajemo vrijednoati funkcije
#define IP    100    // Broj tocaka u kojima racunamo vrijenosti interpolacijom

// Glavni program
int main(void){

  // Deklaracija varijabli
  int i;
  char  dummy[MAXSTR];
  float x_tmp, y_tmp;
  float a, b, x, fx, y_p, y_r, y_s, dy_p, dy_r;
  float *xa, *ya, *fdd, fda, fdb;
  FILE  *fin, *fout;

  // POMOCNI DIO PRORAMA: Deklaracija pomocnih funkcija
  float f(float), fd(float);

  // Definicije polja
  xa=vector(1,NP);  // Polje x koordinata
  ya=vector(1,NP);  // Polje vrijednosti f(x)
  fdd=vector(1,NP); // Polje drugih derivacija potrebnih za spline

  // Velicina intervala na koejm racunamo vrijednosti
  a = -1.0;
  b =  3.0;
  printf("Racunanje f(x) u %i tocaka iz intervala [%f,%f].\n",IP, a, b );

  // POMOCNI DIO PROGRAMA:
  // Inace se tocke ucitavaju iz datoteke pa ovaj dio koda sluzi samo
  // za generiranje ulazne datoteke s vrijednostima ( x, f(x) )
  if ((fin = fopen("in_x_fx.txt","w")) == NULL){
    nrerror("Ulazna datoteka za interpolaciju ne moze biti pohranjena!\n");
    system("PAUSE");
    return 0;
  }
  else {
    fprintf(fin, "%32s", "#             x             f(x)\n");
    fprintf(fin, "%32s", "#  ------------    -------------\n");
    for (i=0;i<NP;i++) {
      x_tmp = a+i*(b-a)/(NP-1);
      y_tmp = f(x_tmp);
      fprintf(fin, "%15.6f  %15.6f \n", x_tmp, y_tmp);
    }
    fclose (fin);
  }

  // Ucitavanje podataka ( x, f(x) ) potrebnih za interpolaciju
  if ((fin = fopen("in_x_fx.txt","r")) == NULL){
    nrerror("Ulazna datoteka za interpolaciju ne moze biti pohranjena!\n");
    return 0;
  }
  else {
    fgets(dummy,MAXSTR,fin);
    fgets(dummy,MAXSTR,fin);
    for (i=0;i<NP;i++) {
      fscanf(fin,"%f %f",&xa[i], &ya[i]);
    }
    fclose (fin);
  }
  
  //Interpolacija u 100 tocaka na razlicite nacine
  if ((fout = fopen("out_x_finter.txt","w")) == NULL) {
    nrerror("Ulazna datoteka za interpolaciju nije pohranjena!\n\n");
	return 0;
	}
  else {
    fprintf(fout,"%52s","#        x     polinomna    racionalna        spline\n");
    fprintf(fout,"%52s","# --------  ------------  ------------  ------------\n");
    fda=fd(a);
    fdb=fd(b);
    spline(xa,ya,NP,fda,fdb,fdd);
    for (i=0;i<100;i++) {
      x=a+i*(b-a)/(100-1);
      fx=f(x);
      polint(xa,ya,NP,x,&y_p,&dy_p);
      ratint(xa,ya,NP,x,&y_r,&dy_r);
      splint(xa,ya,fdd,NP,x,&y_s);
      fprintf(fout,"%10.6f  %12.6f  %12.6f  %12.6f\n", x, y_p, y_r, y_s);
    }
  }

  printf("\n*************************************************************");
  printf("\n* Numericka rjesenja pohranjena su u izlaznim datotekama!\n");
  printf("\n*************************************************************\n");
  fclose (fout);

  // Oslobadjanje memorije po zavrsetku
  free_vector(ya,1,NP);
  free_vector(xa,1,NP);
  system("PAUSE");
  return 0;
}

// POMOCNI DIO PROGRAMA
// Inace se tocke za interpolaciju ucitavaju iz neke datoteke
// Radi jednostavnosti ovdje ih generiramo funkcijom unutar programa 
float f(float x){
  float f;
  f = x*x;
  return f;
}

// POMOCNI DIO PROGRAMA
// Derivacija funkcije f
// Potrebna zbog podataka za spline interpolaciju
float fd(float x){
  float fd;
  fd =  2.*x;
  printf("%f\n",fd);
  system("PAUSE");
  return fd;
}
